;Forbidden Four Multicart
;Copyright 2007 Damian Yerrick
;
;This work is provided 'as-is', without any express or implied
;warranty.  In no event will the authors be held liable for any
;damages arising from the use of this work.
;
;"Source" refers to the preferred form for making changes to a work.
;
;Permission is granted to anyone to use this work for any purpose,
;including commercial applications, and to alter it and redistribute
;it freely, subject to the following restrictions:
;
;  1. The origin of this software must not be misrepresented; you must
;     not claim that you wrote the original work. If you use this work
;     in a product, an acknowledgment in the product documentation
;     would be appreciated but is not required.
;  2. Altered source versions must be plainly marked as such, and must
;     not be misrepresented as being the original work.
;  3. This notice may not be removed or altered from any source
;     distribution.

.include "nes.h"

MENU_BANK = 14
MENU_CHR_BANK = 12 | $20
ARROW_TILE = 14

.import jumpToPRG
.import __RAMCODE_RUN__, __RAMCODE_LOAD__, __RAMCODE_SIZE__

.segment "INESHDR"
  .byt "NES",$1A
  .byt $01, $01, $00, $00
  .res 8, $00

.segment "GAMETABLE"

vblanked = $10
selY = $11
curPad = $12
lastPad = $13
newPresses = $14

gameTable:

; Mario Bros.
  .byt 10      ; CHR bank
  .res 3
  .byt %00000  ; 32 KiB bankswitching (set by menu)
  .byt 10|$10  ; PRG bank, disable WRAM
  .addr $0000  ; Reset vector (set by menu)

; Ice Climber
  .byt 12      ; CHR bank
  .res 3
  .byt %00000  ; 32 KiB bankswitching (set by menu)
  .byt 12|$10  ; PRG bank, disable WRAM
  .addr $0000  ; Reset vector (set by menu)

; Super Mario Bros.
  .byt 10|$20  ; CHR bank
  .res 3
  .byt %00000  ; 32 KiB bankswitching (set by menu)
  .byt 8|$10   ; PRG bank, disable WRAM
  .addr $0000  ; Reset vector (set by menu)

; The Legend of Zelda
  .byt 10      ; CHR bank
  .res 3
  .byt %01111  ; 16 KiB bankswitching
  .byt 7       ; PRG bank
  .addr $E440  ; Reset vector

.segment "BOOTCODE"
reset:
  ldx #0
  stx PPUCTRL
  stx PPUMASK
  ldy #<__RAMCODE_SIZE__
:
  lda __RAMCODE_LOAD__,x
  sta __RAMCODE_RUN__,x
  inx
  dey
  bne :-

  lda #MENU_CHR_BANK
  sta $7F8
  lda #%01011  ; inverted CHR mapping
  sta $7FC
  lda #MENU_BANK
  sta $7FD
  lda #<menuMain
  sta $7FE
  lda #>menuMain
  sta $7FF
  jmp jumpToPRG

.segment "VECTORS"
  .addr nmi, menuMain, 0

.segment "CODE"
menuMain:
  sei
  cld
  ldx #0
  stx PPUCTRL
  stx PPUMASK
  stx vblanked
  stx selY
  dex
  txs
  stx lastPad
  ldx #$0F
  stx SNDCHN
:
  lda PPUSTATUS
  bpl :-
:
  lda PPUSTATUS
  bpl :-

  ldx #$3F
  stx PPUADDR
  ldx #0
  stx PPUADDR
  ldx #8
:
  lda #$1a
  sta PPUDATA
  lda #$0F
  sta PPUDATA
  lda #$18
  sta PPUDATA
  lda #$28
  sta PPUDATA
  dex
  bne :-

  ldx #$20
  stx PPUADDR
  ldy #0
  sty PPUADDR
  ldx #<menuData
  stx 0
  ldx #>menuData
  stx 1
  ldx #4
@drawMenu:
  lda (0),y
  sta PPUDATA
  iny
  bne @drawMenu
  inc 1
  dex
  bne @drawMenu


  lda #$F0
@clearSprites:
  sta OAM,x
  inx
  inx
  inx
  inx
  bne @clearSprites

  lda #ARROW_TILE
  sta OAM+1
  lda #0
  sta OAM+2
  lda #32
  sta OAM+3

mainLoop:
  lda #$80
  sta PPUCTRL
  lda selY
  asl a
  asl a
  asl a
  adc #159
  sta OAM+0

@waitVBL:
  lda vblanked
  beq @waitVBL

  lda #0
  sta vblanked
  sta OAMADDR
  ldx #>OAM
  stx OAM_DMA
  sta PPUSCROLL
  sta PPUSCROLL
  lda #$80
  sta PPUCTRL
  lda #%00011110
  sta PPUMASK

  ; Read the joypad
  ldx #1
  stx P1
  dex
  stx P1
  ldx #8
@readPad:
  lda P1
  lsr a
  rol curPad
  dex
  bne @readPad

  lda lastPad
  eor #$FF
  and curPad
  sta newPresses
  lda curPad
  sta lastPad

  ; Is up pressed?
  lda newPresses
  and #$08
  beq @notUp
  dec selY
  bpl @notUpWrap
  lda #3
  sta selY
@notUpWrap:

@notUp:
  ; Is down pressed?
  lda newPresses
  and #$04
  beq @notDown
  inc selY
  lda selY
  cmp #4
  bcc @notDownWrap
  lda #0
  sta selY
@notDownWrap:

@notDown:

  ; Is down pressed?
  lda newPresses
  and #$90
  beq @notStart
  jmp run
@notStart:


  jmp mainLoop

; Now we run the game.
run:
  lda #$8F
  sta $4000
  lda #$08
  sta $4001
  lda #$80
  sta $4002
  lda #$08
  sta $4003

; Copy jumptable into RAM to run game
  lda selY
  asl a
  asl a
  asl a
  tay
  ldx #0
:
  lda $C000,y
  iny
  sta $07F8,x
  inx
  cpx #8
  bne :-

  jmp jumpToPRG
:
  jmp :-



nmi:
  inc vblanked
  rti

.segment "RODATA"
menuData:
  .incbin "dxinglogo.nam"
