;Forbidden Four Multicart
;Copyright 2007 Damian Yerrick
;
;This work is provided 'as-is', without any express or implied
;warranty.  In no event will the authors be held liable for any
;damages arising from the use of this work.
;
;"Source" refers to the preferred form for making changes to a work.
;
;Permission is granted to anyone to use this work for any purpose,
;including commercial applications, and to alter it and redistribute
;it freely, subject to the following restrictions:
;
;  1. The origin of this software must not be misrepresented; you must
;     not claim that you wrote the original work. If you use this work
;     in a product, an acknowledgment in the product documentation
;     would be appreciated but is not required.
;  2. Altered source versions must be plainly marked as such, and must
;     not be misrepresented as being the original work.
;  3. This notice may not be removed or altered from any source
;     distribution.

.p02
.include "nes.h"

.segment "RAMCODE"
.export jumpToPRG
;;
; Copies a program's CHR into CHR RAM, sets up bankswitch mode, and
; jumps to the program.
; $07F8: Source bank for initial CHR RAM
; 76543210
;   | ||||
;   | ++++- MMC1 PRG bank to load CHR from
;   +------ 0: lower half ($8000-$9FFF);
;           1: upper half ($A000-$BFFF)
; $07FC: MMC1PRG value
; 76543210
;    |||||
;    |||++- Mirroring (2: vertical; 3: horizontal)
;    |++--- PRG bank mode
;    |      (0/1: bank & ~1 in $8000 and bank | 1 in $C000;
;    |       2: first bank in $8000 and bank in $C000;
;    |       3: switchable in $8000 and last bank in $C000)
;    +----- CHR bank mode (0: 8 KiB)
; $07FD: MMC1PRG value
; 76543210
;     ||||
;     ++++- MMC1 PRG bank
; $07FE: Reset vector
jumpToPRG:

  ; Prepare CHR RAM for overwriting
  ldy #0
  sty PPUCTRL
  sty PPUMASK
  sty PPUADDR
  sty PPUADDR

  ; Set MMC1 mode for copying VRAM.
  lda #%01111
  jsr setMMC1CTRL

  ; Set the source bank
  lda $07F8
  and #$0F
  ora #$10
  jsr setMMC1PRG

  ; Set the source address
  lda $07F8
  and #$20
  ora #$80
  sta 1
  ; y is still 0
  sty 0

  ldx #8 * 4  ; 8 KiB times 4 quarter KiB pages

@copyLoop:
  lda (0),y
  sta PPUDATA
  iny
  bne @copyLoop
  inc 1
  dex
  bne @copyLoop

  ; Set MMC1 mode for running program
  lda $07FC
  jsr setMMC1CTRL

  ; Set PRG bank
  lda $07FD
  jsr setMMC1PRG

  ; Set program counter
  jmp ($07FE)

setMMC1CTRL:
  sta MMC1CTRL
  lsr a
  sta MMC1CTRL
  lsr a
  sta MMC1CTRL
  lsr a
  sta MMC1CTRL
  lsr a
  sta MMC1CTRL
  rts

setMMC1PRG:
  sta MMC1PRG
  lsr a
  sta MMC1PRG
  lsr a
  sta MMC1PRG
  lsr a
  sta MMC1PRG
  lsr a
  sta MMC1PRG
  rts

